// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/driver/adadio.h $
// $Rev: 50710 $
// $Date: 2022-03-08 15:35:48 -0600 (Tue, 08 Mar 2022) $

// ADADIO: Device Driver: header file

#ifndef	__ADADIO_H__
#define	__ADADIO_H__

#include "gsc_common.h"
#include "gsc_pci9080.h"



// macros *********************************************************************

#define	ADADIO_BASE_NAME					"adadio"

// IOCTL command codes
#define	ADADIO_IOCTL_REG_READ				OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	ADADIO_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	ADADIO_IOCTL_REG_MOD				OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	ADADIO_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	ADADIO_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	ADADIO_IOCTL_AUTO_CALIBRATE			OS_IOCTL   ( 5)
#define	ADADIO_IOCTL_RX_IO_MODE				OS_IOCTL_RW( 6,  4, s32)
#define	ADADIO_IOCTL_RX_IO_TIMEOUT			OS_IOCTL_RW( 7,  4, s32)
#define	ADADIO_IOCTL_AIN_BUF_CLEAR			OS_IOCTL   ( 8)
#define	ADADIO_IOCTL_AIN_BUF_SIZE			OS_IOCTL_RW( 9,  4, s32)
#define	ADADIO_IOCTL_AIN_BUF_STS			OS_IOCTL_R (10,  4, s32)
#define	ADADIO_IOCTL_AIN_CHAN_LAST			OS_IOCTL_RW(11,  4, s32)
#define	ADADIO_IOCTL_AIN_MODE				OS_IOCTL_RW(12,  4, s32)
#define	ADADIO_IOCTL_AIN_NRATE				OS_IOCTL_RW(13,  4, s32)
#define	ADADIO_IOCTL_AIN_TRIGGER			OS_IOCTL   (14)
#define	ADADIO_IOCTL_AOUT_CH_0_WRITE		OS_IOCTL_RW(15,  4, s32)
#define	ADADIO_IOCTL_AOUT_CH_1_WRITE		OS_IOCTL_RW(16,  4, s32)
#define	ADADIO_IOCTL_AOUT_CH_2_WRITE		OS_IOCTL_RW(17,  4, s32)
#define	ADADIO_IOCTL_AOUT_CH_3_WRITE		OS_IOCTL_RW(18,  4, s32)
#define	ADADIO_IOCTL_AOUT_ENABLE			OS_IOCTL_RW(19,  4, s32)
#define	ADADIO_IOCTL_AOUT_STROBE			OS_IOCTL   (20)
#define	ADADIO_IOCTL_AOUT_STROBE_ENABLE		OS_IOCTL_RW(21,  4, s32)
#define	ADADIO_IOCTL_DATA_FORMAT			OS_IOCTL_RW(22,  4, s32)
#define	ADADIO_IOCTL_DIO_PIN_READ			OS_IOCTL_R (23,  4, s32)
#define	ADADIO_IOCTL_DIO_PIN_WRITE			OS_IOCTL_RW(24,  4, s32)
#define	ADADIO_IOCTL_DIO_PORT_DIR			OS_IOCTL_RW(25,  4, s32)
#define	ADADIO_IOCTL_DIO_PORT_READ			OS_IOCTL_R (26,  4, s32)
#define	ADADIO_IOCTL_DIO_PORT_WRITE			OS_IOCTL_RW(27,  4, s32)
#define	ADADIO_IOCTL_IRQ_SEL				OS_IOCTL_RW(28,  4, s32)
#define	ADADIO_IOCTL_LOOPBACK_CHANNEL		OS_IOCTL_RW(29,  4, s32)
#define	ADADIO_IOCTL_RX_IO_ABORT			OS_IOCTL_R (30,  4, s32)
#define	ADADIO_IOCTL_WAIT_EVENT				OS_IOCTL_RW(31, 28, gsc_wait_t)
#define	ADADIO_IOCTL_WAIT_CANCEL			OS_IOCTL_RW(32, 28, gsc_wait_t)
#define	ADADIO_IOCTL_WAIT_STATUS			OS_IOCTL_RW(33, 28, gsc_wait_t)
#define	ADADIO_IOCTL_AIN_BUF_ENABLE			OS_IOCTL_RW(34,  4, s32)



//*****************************************************************************
// ADADIO_IOCTL_REG_READ
// ADADIO_IOCTL_REG_WRITE
// ADADIO_IOCTL_REG_MOD
//
#define	ADADIO_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	ADADIO_GSC_BRR						GSC_REG_ENCODE(GSC_REG_TYPE_ALT, s32, 0) // Board Revision Register
#define	ADADIO_GSC_BCR						ADADIO_REG_ENCODE(4,0x00)// Board Control Register
#define	ADADIO_GSC_DIOPR					ADADIO_REG_ENCODE(4,0x04)// Digital I/O Port Register
#define	ADADIO_GSC_AOC0R					ADADIO_REG_ENCODE(4,0x08)// Analog Output Ch 0 Register
#define	ADADIO_GSC_AOC1R					ADADIO_REG_ENCODE(4,0x0C)// Analog Output Ch 1 Register
#define	ADADIO_GSC_AOC2R					ADADIO_REG_ENCODE(4,0x10)// Analog Output Ch 2 Register
#define	ADADIO_GSC_AOC3R					ADADIO_REG_ENCODE(4,0x14)// Analog Output Ch 3 Register
#define	ADADIO_GSC_AIDR						ADADIO_REG_ENCODE(4,0x18)// Analog Input Data Register
#define	ADADIO_GSC_SRR						ADADIO_REG_ENCODE(4,0x1C)// Sample Rate Register

//*****************************************************************************
// ADADIO_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	ADADIO_QUERY_AUTO_CAL_MS,	// Max auto-cal period in ms.
	ADADIO_QUERY_COUNT,			// How many query options are supported?
	ADADIO_QUERY_DEVICE_TYPE,	// Value from gsc_dev_type_t
	ADADIO_QUERY_DMDMA,			// Is Demand Mode DMA supported?
	ADADIO_QUERY_FIFO_SIZE,		// FIFO depth in 32-bit samples
	ADADIO_QUERY_FSAMP_MAX,		// The maximum sample rate per channel.
	ADADIO_QUERY_FSAMP_MIN,		// The minimum sample rate per channel.
	ADADIO_QUERY_INIT_MS,		// Max initialize period in ms.
	ADADIO_QUERY_MASTER_CLOCK,	// Master clock frequency
	ADADIO_QUERY_NRATE_MASK,	// Mask of valid Nrate bits.
	ADADIO_QUERY_NRATE_MAX,		// Maximum rate generator Nrate value.
	ADADIO_QUERY_NRATE_MIN,		// Maximum rate generator Nrate value.
	ADADIO_QUERY_VRANGE,		// Voltage ranges supported.
	ADADIO_QUERY_LAST			// The last query index number.
} adadio_query_t;

#define	ADADIO_IOCTL_QUERY_ERROR			(-1)

// ADADIO_QUERY_VRANGE
#define	ADADIO_VRANGE_2_5					2	// +-2.5
#define	ADADIO_VRANGE_5						1	// +-5
#define	ADADIO_VRANGE_10					0	// +-10

//*****************************************************************************
// ADADIO_IOCTL_INITIALIZE					BCR D31
//
// Parameter:	None.

//*****************************************************************************
// ADADIO_IOCTL_AUTO_CALIBRATE				BCR D28, D5
//
// Parameter:	None.

//*****************************************************************************
// ADADIO_IOCTL_RX_IO_MODE
//
//	Parameter:	s32*
//		Pass in any of the gsc_io_mode_t options, or
//		-1 to read the current setting.
#define	ADADIO_IO_MODE_DEFAULT				GSC_IO_MODE_PIO

//*****************************************************************************
// ADADIO_IOCTL_RX_IO_TIMEOUT (in seconds)
//
//	Parameter:	s32*
//		Pass in any value from the minimim to the maximim, the infinite option,
//		or -1 to read the current setting. The value -1 is returned if the
//		feature is not supported, but this should never happen.
#define	ADADIO_IO_TIMEOUT_DEFAULT			10
#define	ADADIO_IO_TIMEOUT_NO_SLEEP			0
#define	ADADIO_IO_TIMEOUT_MIN				0
#define	ADADIO_IO_TIMEOUT_MAX				GSC_IO_TIMEOUT_MAX
#define	ADADIO_IO_TIMEOUT_INFINITE			GSC_IO_TIMEOUT_INFINITE

//*****************************************************************************
// ADADIO_IOCTL_AIN_BUF_CLEAR				BCR D11
//
//	Parameter:	None.

//*****************************************************************************
// ADADIO_IOCTL_AIN_BUF_SIZE				BCR D7-D10
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_AIN_BUF_SIZE_1				0
#define	ADADIO_AIN_BUF_SIZE_2				1
#define	ADADIO_AIN_BUF_SIZE_4				2
#define	ADADIO_AIN_BUF_SIZE_8				3
#define	ADADIO_AIN_BUF_SIZE_16				4
#define	ADADIO_AIN_BUF_SIZE_32				5
#define	ADADIO_AIN_BUF_SIZE_64				6
#define	ADADIO_AIN_BUF_SIZE_128				7
#define	ADADIO_AIN_BUF_SIZE_256				8
#define	ADADIO_AIN_BUF_SIZE_512				9
#define	ADADIO_AIN_BUF_SIZE_1024			10
#define	ADADIO_AIN_BUF_SIZE_2048			11
#define	ADADIO_AIN_BUF_SIZE_4096			12
#define	ADADIO_AIN_BUF_SIZE_8192			13
#define	ADADIO_AIN_BUF_SIZE_16384			14
#define	ADADIO_AIN_BUF_SIZE_32768			15

//*****************************************************************************
// ADADIO_IOCTL_AIN_BUF_STS					BCR D20-D22
//
//	Parameter:	s32*
//		One of the following values will be returned.
#define	ADADIO_AIN_BUF_STS_EMPTY			0
#define	ADADIO_AIN_BUF_STS_ALMOST_EMPTY		1
#define	ADADIO_AIN_BUF_STS_HALF_FULL		2
#define	ADADIO_AIN_BUF_STS_FULL				3

//*****************************************************************************
// ADADIO_IOCTL_AIN_CHAN_LAST				BCR D15-D17
//
//	Parameter:	s32*
//		Pass in any valid value from zero to the 7, or
//		-1 to read the current setting.

//*****************************************************************************
// ADADIO_IOCTL_AIN_MODE					BCR D0-D2
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_AIN_MODE_SE_CONT				0
#define	ADADIO_AIN_MODE_SE_BURST			1
#define	ADADIO_AIN_MODE_DIFF_CONT			2
#define	ADADIO_AIN_MODE_DIFF_BURST			3
#define	ADADIO_AIN_MODE_LB_TEST				4
#define	ADADIO_AIN_MODE_VREF_TEST			5
#define	ADADIO_AIN_MODE_ZERO_TEST			7

//*****************************************************************************
// ADADIO_IOCTL_AIN_NRATE					SRR D0-D15
//
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		At this time the valid range is 100-0xFFFF.

//*****************************************************************************
// ADADIO_IOCTL_AIN_TRIGGER					BCR D30
//
// Parameter:	None.

//*****************************************************************************
// ADADIO_IOCTL_AOUT_CH_0_WRITE				AOC0R D0-D15
// ADADIO_IOCTL_AOUT_CH_1_WRITE				AOC1R D0-D15
// ADADIO_IOCTL_AOUT_CH_2_WRITE				AOC2R D0-D15
// ADADIO_IOCTL_AOUT_CH_3_WRITE				AOC3R D0-D15
//
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		The valid range is 0-0xFFFF.

//*****************************************************************************
// ADADIO_IOCTL_AOUT_ENABLE					BCR D18
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_AOUT_ENABLE_NO				0
#define	ADADIO_AOUT_ENABLE_YES				1

//*****************************************************************************
// ADADIO_IOCTL_AOUT_STROBE					BCR D29
//
// Parameter:	None.

//*****************************************************************************
// ADADIO_IOCTL_AOUT_STROBE_ENABLE			BCR D19
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_AOUT_STROBE_ENABLE_NO		0
#define	ADADIO_AOUT_STROBE_ENABLE_YES		1

//*****************************************************************************
// ADADIO_IOCTL_DATA_FORMAT					BCR D6
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_DATA_FORMAT_2S_COMP			0	// Twos Compliment
#define	ADADIO_DATA_FORMAT_OFF_BIN			1	// Offset Binary

//*****************************************************************************
// ADADIO_IOCTL_DIO_PIN_READ				DIOPR D8
// ADADIO_IOCTL_DIO_PIN_WRITE				DIOPR D9
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_DIO_PIN_CLEAR				0
#define	ADADIO_DIO_PIN_SET					1

//*****************************************************************************
// ADADIO_IOCTL_DIO_PORT_DIR				DIOPR D10
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_DIO_PORT_DIR_INPUT			0
#define	ADADIO_DIO_PORT_DIR_OUTPUT			1

//*****************************************************************************
// ADADIO_IOCTL_DIO_PORT_READ				DIOPR D0-D7
//
//	Parameter:	s32*
//		The value returned will be from 0x00 to 0xFF.

//*****************************************************************************
// ADADIO_IOCTL_DIO_PORT_WRITE				DIOPR D0-D7
//
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current port value.
//		The valid range is 0x00 to 0xFF.

//*****************************************************************************
// ADADIO_IOCTL_IRQ_SEL						BCR D23-D25, D26
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_IRQ_INIT_DONE				0
#define	ADADIO_IRQ_AUTO_CAL_DONE			1
#define	ADADIO_IRQ_AIN_BUF_EMPTY			2
#define	ADADIO_IRQ_AIN_BUF_HALF_FULL		3
#define	ADADIO_IRQ_AIN_BUF_FULL				4
#define	ADADIO_IRQ_AIN_BURST_DONE			5
#define	ADADIO_IRQ_AOUT_STROBE_DONE			6

//*****************************************************************************
// ADADIO_IOCTL_LOOPBACK_CHANNEL			BCR D3-D4
//
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		The valid range is 0 to 3.

//*****************************************************************************
// ADADIO_IOCTL_RX_IO_ABORT
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	ADADIO_IO_ABORT_NO					0
#define	ADADIO_IO_ABORT_YES					1

//*****************************************************************************
// ADADIO_IOCTL_WAIT_EVENT					all fields must be valid
// ADADIO_IOCTL_WAIT_CANCEL					fields need not be valid
// ADADIO_IOCTL_WAIT_STATUS					fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	ADADIO_WAIT_GSC_INIT_DONE			0x0001
#define	ADADIO_WAIT_GSC_AUTO_CAL_DONE		0x0002
#define	ADADIO_WAIT_GSC_AIN_BUF_EMPTY		0x0004
#define	ADADIO_WAIT_GSC_AIN_BUF_HALF_FULL	0x0008
#define	ADADIO_WAIT_GSC_AIN_BUF_FULL		0x0010
#define	ADADIO_WAIT_GSC_AIN_BURST_DONE		0x0020
#define	ADADIO_WAIT_GSC_AOUT_STROBE_DONE	0x0040
#define	ADADIO_WAIT_GSC_ALL					0x007F
// gsc_wait_t.alt flags
#define	ADADIO_WAIT_ALT_ALL					0x0000
// gsc_wait_t.io
#define	ADADIO_WAIT_IO_RX_ABORT				0x0001
#define	ADADIO_WAIT_IO_RX_DONE				0x0002
#define	ADADIO_WAIT_IO_RX_ERROR				0x0004
#define	ADADIO_WAIT_IO_RX_TIMEOUT			0x0008
#define	ADADIO_WAIT_IO_ALL					0x000F

//*****************************************************************************
// ADADIO_IOCTL_AIN_BUF_ENABLE				BCR D11
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	ADADIO_AIN_BUF_ENABLE_YES			0
#define	ADADIO_AIN_BUF_ENABLE_NO			1	// also clears



#endif
