// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/driver/irq.c $
// $Rev: 44398 $
// $Date: 2019-02-06 13:30:22 -0600 (Wed, 06 Feb 2019) $

// ADADIO: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	#define	BCR_IRQ_REQUEST	0x4000000L

	u32	bcr;
	u32	req;

	bcr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_bcr_32);

	if (bcr & BCR_IRQ_REQUEST)
	{
		// Clear the interrupt.
		bcr	^= BCR_IRQ_REQUEST;
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, bcr);

		// Resume any waiting threads.
		req	= GSC_FIELD_DECODE(bcr, 25, 23);

		switch (req)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case ADADIO_IRQ_INIT_DONE:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_INIT_DONE);
				break;

			case ADADIO_IRQ_AUTO_CAL_DONE:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AUTO_CAL_DONE);
				break;

			case ADADIO_IRQ_AIN_BUF_EMPTY:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AIN_BUF_EMPTY);
				break;

			case ADADIO_IRQ_AIN_BUF_HALF_FULL:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AIN_BUF_HALF_FULL);
				break;

			case ADADIO_IRQ_AIN_BUF_FULL:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AIN_BUF_FULL);
				break;

			case ADADIO_IRQ_AIN_BURST_DONE:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AIN_BURST_DONE);
				break;

			case ADADIO_IRQ_AOUT_STROBE_DONE:

				gsc_wait_resume_irq_gsc(dev, ADADIO_WAIT_GSC_AOUT_STROBE_DONE);
				break;
		}
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_bcr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);

	gsc_irq_destroy(dev);
}


