// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/samples/aout/main.c $
// $Rev: 52457 $
// $Date: 2023-01-30 13:50:49 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values	desc
		{	GSC_ARG_S32_FLAG,	&args->chan_0,			"-0",		{ 1 },	"Output on channel 0 (default: all)."				},
		{	GSC_ARG_S32_FLAG,	&args->chan_1,			"-1",		{ 1 },	"Output on channel 1 (default: all)."				},
		{	GSC_ARG_S32_FLAG,	&args->chan_2,			"-2",		{ 1 },	"Output on channel 2 (default: all)."				},
		{	GSC_ARG_S32_FLAG,	&args->chan_3,			"-3",		{ 1 },	"Output on channel 3 (default: all)."				},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },	"Continue testing until an error occurs."			},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },	"Continue testing even if errors occur."			},
		{	GSC_ARG_S32_FLAG,	&args->decimal,			"-d",		{ 1 },	"Use decimal format for saved data."				},
		{	GSC_ARG_S32_FLAG,	&args->decimal,			"-h",		{ 0 },	"Use hexidecimal format for saved data (default)."	},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },	"Continue testing for up to # minutes."				},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },	"Continue testing for up to # iterations."			},
		{	GSC_ARG_S32_MIN,	&args->seconds,			"-s",		{ 1 },	"Generate output for # seconds."					},
		{	GSC_ARG_S32_FLAG,	&args->save_txt,		"-save",	{ 1 },	"Save the pattern data to output.txt."				},
		{	GSC_ARG_S32_FLAG,	&args->save_c,			"-savec",	{ 1 },	"Save the pattern data in C format to output.txt."	},
		{	GSC_ARG_S32_FLAG,	&args->strobe,			"-S",		{ 1 },	"Output data using the output strobe feature."		},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },	"The zero based index of the device to access."		}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"aout",
		/* desc	*/	"Analog output sample application.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->seconds	= 15;

	errs	= gsc_args_parse(argc, argv, &set);

	if ((args->chan_0 == 0) &&
		(args->chan_1 == 0) &&
		(args->chan_2 == 0) &&
		(args->chan_3 == 0))
	{
		args->chan_0	= 1;
		args->chan_1	= 1;
		args->chan_2	= 1;
		args->chan_3	= 1;
	}

	if (args->save_c)
		args->save_txt	= 1;

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty			= 0;
	int		ret;
	long	secs;
	time_t	t_limit		= 0;
	time_t	test;
	long	tests		= 0;

	for (;;)	// This is a convenience loop.
	{
		gsc_label_init(26);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		if (errs == 0)
		{
			t_limit	= exec + (args.minute_limit * 60);
			os_id_host();
			_show_time_stamp();
			_show_access_index(&args);
			errs	+= adadio_init_util(1);
			errs	+= os_id_driver(adadio_open, adadio_read, adadio_close);
			errs	+= adadio_count_boards(1, &qty);
		}

		if (qty <= 0)
			args.continuous	= 0;

		// Open access to device

		if (errs == 0)
			errs	= adadio_open_util(args.index, 0, -1, 1, &args.fd);

		if (errs == 0)
		{
			// Device identification

			errs	= adadio_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args);

			// Close device access

			errs	+= adadio_close_util(args.fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


