// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/samples/savedata/perform.c $
// $Rev: 52464 $
// $Date: 2023-01-30 13:54:44 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	char	buf[64];
	int		errs;
	s32		last	= -1;
	int		ret;

	gsc_label("Input Channels");

	ret		= adadio_ioctl(args->fd, ADADIO_IOCTL_AIN_CHAN_LAST,	&last	);
	errs	= ret ? 1 : 0;

	if (errs == 0)
	{
		chans[0]	= last + 1;

		if (last == 0)
			sprintf(buf, "# %ld", (long) last);
		else
			sprintf(buf, "#s 0-%ld", (long) last);

		printf(	"%ld Channel%s  (%s)\n",
				(long) chans[0],
				(chans[0] == 1) ? "" : "s",
				buf);
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	int		got;

	gsc_label("Reading");
	got	= adadio_read(args->fd, _buffer, get);

	if (got < 0)
	{
		errs	= 1;
	}
	else if (got != get)
	{
		errs	= 1;
		printf(	"FAIL <---  (got %ld samples, requested %ld)\n",
				(long) got,
				(long) get);
	}
	else
	{
		errs	= 0;
		printf(	"PASS  (%ld samples)\n",
				(long) get);
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(int chans, int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			if ((l) && ((l % chans) == 0))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}

			i	= fprintf(file, "  %04lX", (unsigned long) (0xFFFFUL & _buffer[l]));

			if (i != 6)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 32;
	int	errs	= 0;
	s32	nrate;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= adadio_config_ai		(args->fd, -1, 1);
	errs	+= adadio_rx_io_mode	(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= adadio_query(args->fd, -1, 1, ADADIO_QUERY_NRATE_MIN, &nrate);
	errs	+= adadio_ain_nrate		(args->fd, -1, 1, nrate, NULL);
	errs	+= _channels(args, &chans);
	errs	+= adadio_ain_buf_clear	(args->fd, -1, 1);
	errs	+= _read_data(args);
	errs	+= _save_data(chans, errs);

	gsc_label_level_dec();
	return(errs);
}



