// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/samples/sbtest/rx_io_timeout.c $
// $Rev: 52466 $
// $Date: 2023-01-30 13:55:32 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	ADADIO_IO_TIMEOUT_MAX,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	(ADADIO_IO_TIMEOUT_MIN + ADADIO_IO_TIMEOUT_MAX) / 2,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	ADADIO_IO_TIMEOUT_MIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	ADADIO_IO_TIMEOUT_NO_SLEEP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	ADADIO_IO_TIMEOUT_DEFAULT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= adadio_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= adadio_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	#define	TIMEOUT_1	2
	#define	TIMEOUT_2	6

	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	TIMEOUT_1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	TIMEOUT_2,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32		data[1];
	long	dt1;
	long	dt2;
	int		errs	= 0;
	int		i;
	time_t	t1;
	time_t	t2;
	time_t	t3;

	errs	+= adadio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_get_list(fd, list_1);
	t1		= time(NULL);

	if (errs == 0)
	{
		i	= adadio_read(fd, data, 4);

		if (i != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. adadio_read(): returned %d, expected 0)\n",
					__LINE__,
					i);
		}
	}

	errs	+= service_ioctl_set_get_list(fd, list_2);
	t2		= time(NULL);

	if (errs == 0)
	{
		i	= adadio_read(fd, data, 4);

		if (i != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. adadio_read(): returned %d, expected 0)\n",
					__LINE__,
					i);
		}
	}

	t3	= time(NULL);
	dt1	= t2 - t1;
	dt2	= t3 - t2;

	if (errs)
	{
	}
	else if ((dt1 < (TIMEOUT_1 - 1)) || (dt1 > (TIMEOUT_1 + 1)))
	{
		errs++;
		printf(	"FAIL <---  (%d. timeout was %ld, expected %ld to %ld)\n",
				__LINE__,
				dt1,
				(long) TIMEOUT_1 - 1,
				(long) TIMEOUT_1 + 1);
	}
	else if ((dt2 < (TIMEOUT_2 - 1)) || (dt2 > (TIMEOUT_2 + 1)))
	{
		errs++;
		printf(	"FAIL <---  (%d. timeout was %ld, expected %ld to %ld)\n",
				__LINE__,
				dt2,
				(long) TIMEOUT_2 - 1,
				(long) TIMEOUT_2 + 1);
	}

	errs	+= adadio_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	rx_io_timeout_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service ADADIO_IOCTL_RX_IO_TIMEOUT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_io_timeout_test(int fd)
{
	int	errs	= 0;

	gsc_label("ADADIO_IOCTL_RX_IO_TIMEOUT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


