// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_ain_buf_sts.c $
// $Rev: 48529 $
// $Date: 2020-12-02 12:35:48 -0600 (Wed, 02 Dec 2020) $

// ADADIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	adadio_ain_buf_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the ADADIO_IOCTL_AIN_BUF_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int adadio_ain_buf_sts(int fd, int index, int verbose, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;
	s32		set		= -1;

	if (verbose)
		gsc_label_index("Input Buffer Status", index);

	ret		= adadio_ioctl(fd, ADADIO_IOCTL_AIN_BUF_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case ADADIO_AIN_BUF_STS_EMPTY:

			strcpy(buf, "Empty");
			break;

		case ADADIO_AIN_BUF_STS_ALMOST_EMPTY:

			strcpy(buf, "Almost Empty");
			break;

		case ADADIO_AIN_BUF_STS_HALF_FULL:

			strcpy(buf, "Half Full");
			break;

		case ADADIO_AIN_BUF_STS_FULL:

			strcpy(buf, "Full");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


