// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 52440 $
// $Date: 2023-01-30 13:19:25 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	adadio_query
*
*	Purpose:
*
*		Provide a visual wrapper for the ADADIO_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	adadio_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case ADADIO_QUERY_AUTO_CAL_MS:	ptr	= "Auto-Cal Duration";	break;
		case ADADIO_QUERY_COUNT:		ptr	= "Query Options";		break;
		case ADADIO_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case ADADIO_QUERY_DMDMA:		ptr	= "DMDMA Support";		break;
		case ADADIO_QUERY_FIFO_SIZE:	ptr	= "FIFO Size";			break;
		case ADADIO_QUERY_FSAMP_MAX:	ptr	= "Fsamp Max";			break;
		case ADADIO_QUERY_FSAMP_MIN:	ptr	= "Fsamp Min";			break;
		case ADADIO_QUERY_INIT_MS:		ptr	= "Init. Duration";		break;
		case ADADIO_QUERY_MASTER_CLOCK:	ptr	= "Master Clock";		break;
		case ADADIO_QUERY_NRATE_MASK:	ptr	= "Nrate Mask";			break;
		case ADADIO_QUERY_NRATE_MAX:	ptr	= "Nrate Max";			break;
		case ADADIO_QUERY_NRATE_MIN:	ptr	= "Nrate Min";			break;
		case ADADIO_QUERY_VRANGE:		ptr	= "Voltage Range";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= adadio_ioctl(fd, ADADIO_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case ADADIO_QUERY_COUNT:
		case ADADIO_QUERY_NRATE_MAX:
		case ADADIO_QUERY_NRATE_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case ADADIO_QUERY_NRATE_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case ADADIO_QUERY_MASTER_CLOCK:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case ADADIO_QUERY_AUTO_CAL_MS:
		case ADADIO_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case ADADIO_QUERY_FSAMP_MAX:
		case ADADIO_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case ADADIO_QUERY_DMDMA:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case ADADIO_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case ADADIO_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_ADADIO)
			{
				strcpy(buf, "ADADIO");
			}
			else if (set == GSC_DEV_TYPE_ADADIO2)
			{
				strcpy(buf, "ADADIO2");
			}
			else
			{
				errs	= 1;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case ADADIO_QUERY_VRANGE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case ADADIO_VRANGE_2_5:

					strcpy(buf, "+-2.5V");
					break;

				case ADADIO_VRANGE_5:

					strcpy(buf, "+-5V");
					break;

				case ADADIO_VRANGE_10:

					strcpy(buf, "+-10V");
					break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


